<?php

namespace Drupal\fileslog\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Pager\PagerManagerInterface;
use Drupal\fileslog\FilesLogInterface;
use Drupal\fileslog\FilesLogManagerInterface;
use Drupal\fileslog\Form\FileslogFilterForm;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Renders the fileslog routes.
 */
class FilesLogController extends ControllerBase {

  /**
   * The pager manager.
   *
   * @var \Drupal\Core\Pager\PagerManagerInterface
   */
  protected PagerManagerInterface $pagerManager;

  /**
   * The fileslog manager.
   *
   * @var \Drupal\fileslog\FilesLogManagerInterface
   */
  protected FilesLogManagerInterface $filesLogManager;

  /**
   * Constructs the controller.
   *
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder.
   * @param \Drupal\Core\Pager\PagerManagerInterface $pager_manager
   *   The pager manager.
   * @param \Drupal\fileslog\FilesLogManagerInterface $files_log_manager
   *   The fileslog manager.
   */
  public function __construct(
    FormBuilderInterface $form_builder,
    PagerManagerInterface $pager_manager,
    FilesLogManagerInterface $files_log_manager,
  ) {
    $this->formBuilder = $form_builder;
    $this->pagerManager = $pager_manager;
    $this->filesLogManager = $files_log_manager;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('form_builder'),
      $container->get('pager.manager'),
      $container->get('fileslog.manager'),
    );
  }

  /**
   * View page for each log.
   *
   * @param string $filename
   *   The log's filename.
   *
   * @return array
   *   The render array.
   */
  public function view(string $channel, string $filename): array {
    // We use this string as an identifier for logs without a channel.
    if ($channel === 'fileslog-unknown-channel') {
      $channel = '';
    }

    if (!$log = $this->filesLogManager->getLog($channel, $filename)) {
      throw new NotFoundHttpException();
    }

    return $log->render('canonical');
  }

  /**
   * Renders an overview of the (filtered) logs.
   *
   * @return array
   *   The render array.
   */
  public function overview(Request $request): array {
    $filters = $request->getSession()->get(FileslogFilterForm::SESSION_VALUE_NAME, []);
    $page = $this->pagerManager->findPage();
    $offset = $page * FilesLogInterface::ITEMS_PER_PAGE;
    $logs = $this->filesLogManager->getLogs($filters, FilesLogInterface::ITEMS_PER_PAGE, $offset);
    $total = $this->filesLogManager->getTotalLogsCount();

    $this->pagerManager->createPager($total, FilesLogInterface::ITEMS_PER_PAGE);

    return [
      '#theme' => 'fileslog_overview',
      '#rows' => array_map(function ($entry) {
        return $entry->render('row');
      }, $logs),
      '#summary' => [
        'offset' => $offset + 1,
        'amount' => min($offset + FilesLogInterface::ITEMS_PER_PAGE, $total),
        'total' => $total,
      ],
      '#form' => $this->formBuilder()->getForm(FileslogFilterForm::class),
      '#pager' => [
        '#type' => 'pager',
      ],
    ];
  }

}
